#define stepPin 3 
#define dirPin 2 
#define enablePin 7
#define MicroStep1Pin 6
#define MicroStep2Pin 5
#define MicroStep3Pin 4

#define numMsInOneSec 1000
#define numMicroSecInOneMs 1000
#define stepPulseWidthInMicroSec 2
#define setupTimeInMicroSec 1
//#define timeBetweenStepsInMicroSec (1 * numMicroSecInOneMs)

#define inputBufferSize 128

int  serialCharIn;             
char serialInString[inputBufferSize];  
int  serialInIndex  = 0;

unsigned long timeBetweenInputPollsInMicroSec = ( (unsigned long)(numMsInOneSec /4) * numMicroSecInOneMs ); // 250 = 1/4th of a second
unsigned long timeBetweenStepsInMicroSec = (1 * numMicroSecInOneMs);

unsigned long loopCheck = 0;

boolean lineReady = false;
boolean successfullyParsed = false;
boolean currentDirection = false;
boolean shouldStep = true;

boolean speedChanged = true;

void setCurrentDirection(boolean dir)
{
  if(dir == false)
  {
      digitalWrite(dirPin, LOW);
  } else {
      digitalWrite(dirPin, HIGH);
  }
  currentDirection = dir;
  delayMicroseconds(setupTimeInMicroSec);
}

void changeDirection()
{
  setCurrentDirection(!currentDirection);
}

void enableStepper(int isEnabled)
{
  if(isEnabled)
  {
      digitalWrite(enablePin, LOW); // enable HIGH = stepper driver OFF
  } else {
      digitalWrite(enablePin, HIGH); // enable HIGH = stepper driver OFF
  }
  // wait a few microseconds for the enable to take effect 
  // (That isn't in the spec sheet I just added it for sanity.) 
  delayMicroseconds(2);
}

void takeSingleStep()
{
    digitalWrite(stepPin, LOW);
    delayMicroseconds(stepPulseWidthInMicroSec); 
    digitalWrite(stepPin, HIGH); 
    delayMicroseconds(stepPulseWidthInMicroSec); 
    digitalWrite(stepPin, LOW);
}

void setFullStep()
{
  digitalWrite(MicroStep1Pin, LOW);
  digitalWrite(MicroStep2Pin, LOW);
  digitalWrite(MicroStep3Pin, LOW);
  delayMicroseconds(setupTimeInMicroSec);
}

void setHalfStep()
{
  digitalWrite(MicroStep1Pin, HIGH);
  digitalWrite(MicroStep2Pin, LOW);
  digitalWrite(MicroStep3Pin, LOW);
  delayMicroseconds(setupTimeInMicroSec);
}

void setQuarterStep()
{
  digitalWrite(MicroStep1Pin, LOW);
  digitalWrite(MicroStep2Pin, HIGH);
  digitalWrite(MicroStep3Pin, LOW);
  delayMicroseconds(setupTimeInMicroSec);  
}

void setEighthStep()
{
  digitalWrite(MicroStep1Pin, HIGH);
  digitalWrite(MicroStep2Pin, HIGH);
  digitalWrite(MicroStep3Pin, LOW);
  delayMicroseconds(setupTimeInMicroSec);  
}

void setSixteenthStep()
{
  digitalWrite(MicroStep1Pin, HIGH);
  digitalWrite(MicroStep2Pin, HIGH);
  digitalWrite(MicroStep3Pin, HIGH);
  delayMicroseconds(setupTimeInMicroSec);  
}

void readSerialString () 
{
  int serialCharacter;   
  if(Serial.available()) 
  { 
     while (Serial.available())
     { 
        serialCharacter = Serial.read();
        if(serialInIndex == inputBufferSize-1)
        {
          serialInString[serialInIndex] = '\0';
          // we just keep setting the last character in our buffer and just loose characters
        } else {          
          serialInString[serialInIndex] = serialCharacter;
          serialInIndex++;
        }
        
        if(serialCharacter == '\n')
        {
          lineReady = true;
        }
      }
   }
}  

void serialResponseString() {
   if( lineReady )
   {
     if(successfullyParsed == true)
     {
       Serial.print("String successfully parsed: ");
     } else {
       Serial.print("String unsuccessfully parsed: ");
     }
     
     Serial.print(serialInString);     
     serialInIndex = 0;
     lineReady = false;
     Serial.println();
   }
}

bool makeChangesBasedOnSerial()
{
  boolean output = true;
  String tempString = serialInString;
  String hzStartString = "StepsInHz=";
  if(tempString == "ChangeDirection\n")
  {
    changeDirection();
  } else if (tempString == "EnableStepper\n") {
    enableStepper(true);
  } else if (tempString == "DisableStepper\n") {
    enableStepper(false);
  } else if (tempString == "FullStep\n") {
    setFullStep();
  } else if (tempString == "HalfStep\n") {
    setHalfStep();
  } else if (tempString == "QuarterStep\n") {
    setQuarterStep();
  } else if (tempString == "EighthStep\n") {
    setEighthStep();
  } else if (tempString == "SixteenthStep\n") {
    setSixteenthStep();
  } else if (tempString == "SteppingOn\n") {
    shouldStep = true;
  } else if (tempString == "SteppingOff\n") {
    shouldStep = false;
  } else if (tempString.startsWith( hzStartString ) && tempString.endsWith("\n") ) {
    // must be less than 1,000,000
    String hzstr = tempString.substring(hzStartString.length(), tempString.length() -1);
    char hzCharArray[inputBufferSize];
    hzstr.toCharArray(hzCharArray, inputBufferSize);
//    Serial.println(hzstr);
    long parsed = atol(hzCharArray);
//    Serial.println(parsed);
    if(parsed != 0 && parsed < (numMicroSecInOneMs*numMsInOneSec) )
    {
      timeBetweenStepsInMicroSec = ((unsigned long)numMicroSecInOneMs*numMsInOneSec) / (unsigned long) parsed;
      speedChanged = true;
    } else {
      output = false;
    }
  } else {
    output = false;
  }
  return output;
}


void setup() 
{
  // We set the enable pin to be an output 
  pinMode(enablePin, OUTPUT); // then we set it HIGH so that the board is disabled until we 
  pinMode(stepPin, OUTPUT); 
  pinMode(dirPin, OUTPUT);
  
  pinMode(MicroStep1Pin, OUTPUT);
  pinMode(MicroStep2Pin, OUTPUT);
  pinMode(MicroStep3Pin, OUTPUT);
  
    // get into a known state. 
  enableStepper(false);
  // we set the direction pin in an arbitrary direction.
  setCurrentDirection(false);
  setFullStep();
  
  Serial.begin(9600); 
  Serial.println("Starting stepper dev tester.");
  
  enableStepper(true);
 // we set the direction pin in an arbitrary direction. 
  setCurrentDirection(true);
}

void loop() 
{ 
  int j; 

  readSerialString();
  if( lineReady )
  {
    // ensure string is null terminated    
      serialInString[serialInIndex] = '\0';
      serialInIndex++;
      
    successfullyParsed = makeChangesBasedOnSerial();
    serialResponseString();
  }

  if(shouldStep)
  {
    if(speedChanged)
    {
      loopCheck = (timeBetweenInputPollsInMicroSec/timeBetweenStepsInMicroSec);
      //Serial.println(loopCheck);
      speedChanged = false;
    }
    
    for(j=0; j<loopCheck; j++) 
    { 
      takeSingleStep();
      delayMicroseconds(timeBetweenStepsInMicroSec);
    } 
  }
}

